
/******************************************************************************
 Hafler IRIS Remote Reciever
 Dean Hedin 1/23/2022
 
 This sketch (along with special hardware) implements the basic features of 
 the Hafler IRIS remote using an inexspensive IR remote kit for the Arduino.
  
 This code is targeted to the Arduino Pro Micro (5 volt version 16mhz). If
 a different Arduino is used then pin re-assignmets and other code changes may 
 be required.  Total hardware current consumption is 44 milliamps.

 The IR kit utilized is the "M87" type.  This particular kit was chosen mainly 
 because of it's useful keyboard layout. That is up/dn/right/left arrows and 
 numeric keypad.

 Input selection, Volume, Balance, Mute, and Unmute are implemented.   

 If a different remote is utilized you will have to connect the arduino 
 terminal and press each button and write down the corresponding hex code.  
 The code then modified with the new codes and re-compiled.

 On power up volume is set to it's lowest setting and balance in the center.

 To do:

 1) I've not implemented the special behaviors of the original Hafler with respect 
 to the tape monitoring functions as described in the IRIS owner's manual. Somebody
 might want that but they will have to go at that themselves.
 2) I think I might implements a "return to center" for the balance.  Once you
 change balance on the remote you have no feedback as to where "center" is other than 
 using your ears.  In fact I think I will make it so that pressing the "OK" button
 will center balance and set volume to around 30%.
 3) Might consider using an unused button to control external hardware to "swap in"
 a more modern pre-amp circuit inside the IRIS.
 4) It is not so difficult to implement a remote transmitter based on an Arduino
 that would very similiar to the original IRIS unit.  I just might do that someday.
 5) Might consider using one of the newer IR remote libraries like IRLib2
 6) Might consider saving last used volume, balance, and input to non volatile memory
 for restoration on next startup.
  
 */


#include <IRremote.h>
#include <avr/sleep.h>
#include <avr/power.h>


//FYI -only pins 3,5,6,9,10 may be configured for PWM on Pro Micro.

const int RECV_INPUT_PIN = 16;
const int VOL_OUTPUT_PIN =  9;
const int BAL_OUTPUT_PIN = 10;
const int MUX_LATCH_OUTPUT_PIN = 2;
const int MUX_A_OUTPUT_PIN = 3;
const int MUX_B_OUTPUT_PIN = 4;
const int MUX_C_OUTPUT_PIN = 5;
const int MUTE_OUTPUT_PIN = 6;
const int NORM_OUTPUT_PIN = 7;
const int REM_VB_OUTPUT_PIN = 8;

//Delay hold time ms for latch for MUX_LATCH_OUTPUT_PIN and REM_VB_OUTPUT_PIN
//This goes to the INH pin of the CD4051. If you scope it out it's duration is about 4ms
//when a human selects an output on the front panel.  This width is accomplished with Schmitt trigger
const int HOLD_TIME = 3; 



const unsigned long BAUD_RATE = 9600;

IRrecv irrecv(RECV_INPUT_PIN);

decode_results results;

uint32_t lastvalue = 0;

int incomingByte;

int Vol_Level;
int Bal_Level;
int Vol_Bal_Step;



//MUX table for input selection on Iris         
//INHIBIT C B A   
//0       0 0 0 0 Tuner
//0       0 0 1 1 Phono
//0       0 1 0 2 CD
//0       0 1 1 3 Video
//0       1 0 0 4 Aux1
//0       1 0 1 5 Aux2
//0       1 1 0 6 Tape 1
//0       1 1 1 7 Tape 3
//1       X X X NONE  



void Proc_Set_MUX_LATCH_Low()
{
  digitalWrite(MUX_A_OUTPUT_PIN, HIGH);
  digitalWrite(MUX_B_OUTPUT_PIN, HIGH);
  digitalWrite(MUX_C_OUTPUT_PIN, HIGH);
  digitalWrite(MUX_LATCH_OUTPUT_PIN,HIGH);
}


void Proc_Toggle_VB_Latch()
{
  digitalWrite(REM_VB_OUTPUT_PIN,LOW);
  delay(HOLD_TIME);
  digitalWrite(REM_VB_OUTPUT_PIN,HIGH);
}

void Proc_Sel_Phono() //A1 B0 C0 - Phono
{  
  Serial.print("Select Phono\n");  
  digitalWrite(MUX_LATCH_OUTPUT_PIN,LOW);   
  digitalWrite(MUX_A_OUTPUT_PIN, LOW);
  digitalWrite(MUX_B_OUTPUT_PIN, HIGH);
  digitalWrite(MUX_C_OUTPUT_PIN, HIGH);
  delay(HOLD_TIME);
  Proc_Set_MUX_LATCH_Low();
}
void Proc_Sel_CD()  //A0 B1 C0 - CD
{
  Serial.print("Select CD\n");
  digitalWrite(MUX_LATCH_OUTPUT_PIN,LOW);
  digitalWrite(MUX_A_OUTPUT_PIN, HIGH);
  digitalWrite(MUX_B_OUTPUT_PIN, LOW);
  digitalWrite(MUX_C_OUTPUT_PIN, HIGH);  
  
  Proc_Set_MUX_LATCH_Low();
}
void Proc_Sel_Tuner()  //A0 B0 C0 Tuner
{
  Serial.print("Select Tuner\n");
  digitalWrite(MUX_LATCH_OUTPUT_PIN,LOW); 
  delay(HOLD_TIME); 
  Proc_Set_MUX_LATCH_Low(); 
}
void Proc_Sel_Video() //A1 B1 C0 Video
{
  Serial.print("Select Video\n");
  digitalWrite(MUX_LATCH_OUTPUT_PIN,LOW);
  digitalWrite(MUX_A_OUTPUT_PIN, LOW);
  digitalWrite(MUX_B_OUTPUT_PIN, LOW);
  digitalWrite(MUX_C_OUTPUT_PIN, HIGH); 
  delay(HOLD_TIME);
  Proc_Set_MUX_LATCH_Low();
}
void Proc_Sel_AUX1()  //0 0 1 Aux1
{
  Serial.print("Select AUX1\n");
  digitalWrite(MUX_LATCH_OUTPUT_PIN,LOW);
  digitalWrite(MUX_A_OUTPUT_PIN, HIGH);
  digitalWrite(MUX_B_OUTPUT_PIN, HIGH);
  digitalWrite(MUX_C_OUTPUT_PIN, LOW); 
  delay(HOLD_TIME);
  Proc_Set_MUX_LATCH_Low(); 
}
void Proc_Sel_AUX2() //A1 B0 C1 - Aux2
{
  Serial.print("Select AUX2\n");
  digitalWrite(MUX_LATCH_OUTPUT_PIN,LOW);
  digitalWrite(MUX_A_OUTPUT_PIN, LOW);
  digitalWrite(MUX_B_OUTPUT_PIN, HIGH);
  digitalWrite(MUX_C_OUTPUT_PIN, LOW);  
  delay(HOLD_TIME);
  Proc_Set_MUX_LATCH_Low(); 
}
void Proc_Sel_Tape1()  //A0 B1 C1 - Tape 1
{
  Serial.print("Select Tape1\n");  
  digitalWrite(MUX_LATCH_OUTPUT_PIN,LOW);
  digitalWrite(MUX_A_OUTPUT_PIN, HIGH);
  digitalWrite(MUX_B_OUTPUT_PIN, LOW);
  digitalWrite(MUX_C_OUTPUT_PIN, LOW);  
  delay(HOLD_TIME);
  Proc_Set_MUX_LATCH_Low(); 
}
void Proc_Sel_Tape2()   //A1 B1 C1 - Tape 3
{
  Serial.print("Select Tape2\n");
  digitalWrite(MUX_LATCH_OUTPUT_PIN,LOW);
  digitalWrite(MUX_A_OUTPUT_PIN, LOW);
  digitalWrite(MUX_B_OUTPUT_PIN, LOW);
  digitalWrite(MUX_C_OUTPUT_PIN, LOW);  
  delay(HOLD_TIME);
  Proc_Set_MUX_LATCH_Low(); 
}


//Note:
//Our PWM configuration permits us 800 steps for 0-100% pwm 
//Through the output Opamp we potentially have 0-16v
//But IRIS schematic indicates 0-14v for volume and balance
//Hence 14v/16v * 800 steps = 700 steps  
//So we top out at 699 steps.
//We jump by 2 steps at a time which gives us about 350 clicks.
//This exceeds the original IRIS remote which only had 256 clicks.
void Proc_Vol_Up()
{
  Serial.print("Volume Up :\n");
  Vol_Level=Vol_Level+Vol_Bal_Step;
  if (Vol_Level > 699)
    Vol_Level=699;
  OCR1A = Vol_Level;
  Proc_Toggle_VB_Latch();
  Serial.println(Vol_Level, DEC);
}
void Proc_Vol_Dn()
{
  Serial.print("Volume Down :\n");
  Vol_Level=Vol_Level-Vol_Bal_Step;
  if (Vol_Level < 0)
    Vol_Level=0;
  OCR1A = Vol_Level;
  Proc_Toggle_VB_Latch();
  Serial.println(Vol_Level, DEC);
}
void Proc_Bal_Left()
{
  Serial.print("Balance Left\n");
  Bal_Level=Bal_Level+Vol_Bal_Step;
  if (Bal_Level > 699)
    Bal_Level=699;
  OCR1B = Bal_Level;  
  Proc_Toggle_VB_Latch();
  Serial.println(Bal_Level, DEC);
}
void Proc_Bal_Right()
{
  Serial.print("Balance Right\n");
  Bal_Level=Bal_Level-Vol_Bal_Step;
  if (Bal_Level < 0)
    Bal_Level=0;
  OCR1B = Bal_Level;
  Proc_Toggle_VB_Latch();
  Serial.println(Bal_Level, DEC);
}
void Proc_Normal()
{
  Serial.print("Set Normal\n");
  digitalWrite(NORM_OUTPUT_PIN,LOW);
  delay(HOLD_TIME);
  digitalWrite(NORM_OUTPUT_PIN,HIGH);
}
void Proc_Mute()
{
  Serial.print("Set Mute\n");
  digitalWrite(MUTE_OUTPUT_PIN,LOW);
  delay(HOLD_TIME);
  digitalWrite(MUTE_OUTPUT_PIN,HIGH);
}

void setup()
{
  
  pinMode(RECV_INPUT_PIN, INPUT);
  pinMode(REM_VB_OUTPUT_PIN, OUTPUT);
  pinMode(MUX_A_OUTPUT_PIN, OUTPUT);
  pinMode(MUX_B_OUTPUT_PIN, OUTPUT);
  pinMode(MUX_C_OUTPUT_PIN, OUTPUT);
  pinMode(MUX_LATCH_OUTPUT_PIN, OUTPUT);
  pinMode(MUTE_OUTPUT_PIN, OUTPUT);
  pinMode(NORM_OUTPUT_PIN, OUTPUT);

  
  //Logic level translation inverts the logic.
  //These lines are pulled up to +5v to ensure that the 
  //CMOS level (0,16v) lines to hafler are low at power up.
  digitalWrite(REM_VB_OUTPUT_PIN, HIGH);
  digitalWrite(MUX_A_OUTPUT_PIN, HIGH);
  digitalWrite(MUX_B_OUTPUT_PIN, HIGH);
  digitalWrite(MUX_C_OUTPUT_PIN, HIGH);
  digitalWrite(MUX_LATCH_OUTPUT_PIN, HIGH);
  digitalWrite(MUTE_OUTPUT_PIN, HIGH);
  digitalWrite(NORM_OUTPUT_PIN, HIGH);
  
  pinMode(VOL_OUTPUT_PIN, OUTPUT);
  pinMode(BAL_OUTPUT_PIN, OUTPUT);
  analogWrite(VOL_OUTPUT_PIN, Vol_Level);
  analogWrite(BAL_OUTPUT_PIN, Bal_Level);
  
  Serial.begin(BAUD_RATE);
  Serial.print("Hedin's Hafler IR remote up and running...\n");
  irrecv.enableIRIn(); // Start the receiver


  //consider saving/restoring volume/balance in flash?  
  //Setup and customize the PWM frequency
  
  Vol_Level = 0;    //to be safe start volume at minimum
  Bal_Level = 399;  //and balance mid range 
  Vol_Bal_Step = 2; //changes to 4 if repeat occurs

  TCCR1A = 0;
  TCCR1B = 0;

  TCCR1A = bit (COM1A1) | bit(COM1B1) | bit (WGM11); // Enable PWM outputs for OC1A and OC1B on digital pins 9, 10
  TCCR1B = bit (WGM13) | bit (WGM12) | bit (CS10);  // Set fast PWM and prescaler of 8 on timer 1
  ICR1 = 799; //sets to 20kHz
  OCR1A = Vol_Level; // Set this to change duty cycle 0-799 for pin 9
  OCR1B = Bal_Level; //..for pin 10 
  pinMode(VOL_OUTPUT_PIN, OUTPUT);                               // Set digital pin 9 (D9) to an output
  pinMode(BAL_OUTPUT_PIN, OUTPUT);                              // Set digital pin 10 (D10) to an output

  power_adc_disable(); // disable ADC converter
  power_spi_disable(); // disable SPI
  power_twi_disable();  // disable I2C 
  
}

//"M87" Remote. FYI, another brand remote probably will have different codes
//FFA25D  1
//FF629D  2
//FFE21D  3
//FF22DD  4
//FF02FD  5
//FFC23D  6
//FFE01F  7
//FFA857  8
//FF906F  9
//FF9867  0
//FF6897  *
//FFB04F  #
//FF18E7  Up arrow
//FF4AB5  Dn Arrow
//FF10EF  Left Arrow
//FF5AA5  Right Arrow
//FF38C7 "OK"
//FFFFFFFF Repeat

void loop() 
{  
  if (irrecv.decode(&results)) 
  {
   Serial.println(results.value, HEX);
   if (results.value != 0xFFFFFFFF)
      lastvalue=results.value;
      
   switch (results.value)
    {
      case  0xFFA25D : Proc_Sel_Phono(); break;  //1 Phono
      case  0xFF629D : Proc_Sel_CD(); break;     //2 CD
      case  0xFFE21D : Proc_Sel_Tuner(); break;  //3 Tuner
      case  0xFF22DD : Proc_Sel_Video(); break;  //4 Video
      case  0xFF02FD : Proc_Sel_AUX1(); break;   //5 Aux 1
      case  0xFFC23D : Proc_Sel_AUX2(); break;   //6 Aux 2
      case  0xFFE01F : Proc_Sel_Tape1(); break;  //7 Tape 1
      case  0xFFA857 : Proc_Sel_Tape2(); break;  //8 Tape 2
      case  0xFF18E7 : Proc_Vol_Up(); break;     // Vol up
      case  0xFF4AB5 : Proc_Vol_Dn(); break;     // Vol dn
      case  0xFF10EF : Proc_Bal_Right(); break;  // Bal R
      case  0xFF5AA5 : Proc_Bal_Left(); break;   // Bal L 
      case  0xFFB04F : Proc_Normal(); break;     // normal
      case  0xFF6897 : Proc_Mute(); break;       // mute
      case  0xFFFFFFFF : { //repeat code
                          Vol_Bal_Step = 4; //we step faster...
                          switch(lastvalue)
                          {
                            //repeat only allowed for these functions
                            case  0xFF18E7 : Proc_Vol_Up(); break;     //'q' Vol up
                            case  0xFF4AB5 : Proc_Vol_Dn(); break;     //'a' Vol dn
                            case  0xFF10EF : Proc_Bal_Right(); break;  //'s' Bal R
                            case  0xFF5AA5 : Proc_Bal_Left(); break;   //'w' Bal L
                          }
                          Vol_Bal_Step = 2; //set back
                         } break;
    }
  
   irrecv.resume(); // Receive the next value
  }
  /* 
  //control through serial via terminal for debugging.
  if (Serial.available() > 0) 
  {
    // read the incoming byte:
    incomingByte = Serial.read();

    // say what you got:
    if (incomingByte != 10)
    {
    //Serial.print("I received: ");
    //Serial.println(incomingByte, DEC);
    }
    switch (incomingByte)
    {
      case  49 : Proc_Sel_Phono(); break;  //'1' Phono
      case  50 : Proc_Sel_CD(); break;     //'2' CD
      case  51 : Proc_Sel_Tuner(); break;  //'3' Tuner
      case  52 : Proc_Sel_Video(); break;  //'4' Video
      case  53 : Proc_Sel_AUX1(); break;   //'5' Aux 1
      case  54 : Proc_Sel_AUX2(); break;   //'6' Aux 2
      case  55 : Proc_Sel_Tape1(); break;  //'7' Tape 1
      case  56 : Proc_Sel_Tape2(); break;  //'8' Tape 2
      case  113: Proc_Vol_Up(); break;     //'q' Vol up
      case  97 : Proc_Vol_Dn(); break;     //'a' Vol dn
      case  119: Proc_Bal_Left(); break;   //'w' Bal L
      case  115: Proc_Bal_Right(); break;  //'s' Bal R
      case  110: Proc_Normal(); break;     //'n' normal
      case  109: Proc_Mute(); break;       //'m' mute
    }
    
  }
  */
  
}
